@@ -22,7 +22,14 @@ plot.competing.risk.rfsrc <- function (x, plots.one.page = FALSE, ...) {
   opar <- par(no.readonly = TRUE)
   on.exit(par(opar))
   if (plots.one.page) par(mfrow = c(1,1)) else par(mfrow = c(1,2))
-  matPlot(apply(x$chf, c(2, 3), mean, na.rm = TRUE), "CHF", "CSCHF", pos = 2)
-  matPlot(100 * apply(x$cif, c(2, 3), mean, na.rm = TRUE), "Probability (%)", "CIF", 2)
+  ## plot the results - use OOB whenever possible
+  if (!is.null(x$chf.oob)) {
+    matPlot(apply(x$chf.oob, c(2, 3), mean, na.rm = TRUE), "OOB CHF", "OOB CSCHF", pos = 2)
+    matPlot(100 * apply(x$cif.oob, c(2, 3), mean, na.rm = TRUE), "Probability (%)", "OOB CIF", 2)
+  }
+  else {
+   matPlot(apply(x$chf, c(2, 3), mean, na.rm = TRUE), "CHF", "CSCHF", pos = 2)
+    matPlot(100 * apply(x$cif, c(2, 3), mean, na.rm = TRUE), "Probability (%)", "CIF", 2)
+  } 
 }
 plot.competing.risk <- plot.competing.risk.rfsrc
