@@ -3,6 +3,7 @@ predict.rfsrc <-
            newdata,
            m.target = NULL,
            importance = c(FALSE, TRUE, "none", "permute", "random", "anti"),
+           get.tree = NULL,
            block.size = NULL,
            ensemble = NULL,
            na.action = c("na.omit", "na.impute"),
@@ -25,6 +26,7 @@ predict.rfsrc <-
                                           ensemble = ensemble,
                                           m.target = m.target,
                                           importance = importance,
+                                          get.tree = get.tree,                                          
                                           block.size = block.size,
                                           na.action = na.action,
                                           outcome = outcome,
